using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using VideoCaptureLibrary;

namespace VideoCaptureLibraryDemo
{
    public partial class MainFrm : Form
    {
        private VideoCaptureDevice videoCaptureDevice;
        private bool useCaptureStartTime;
        private DateTime captureStartTime;
        private bool alreadyCaptureTime = false;

        public MainFrm()
        {
            InitializeComponent();
            //Check whether devices connected to PC.
            while (true)
            {
                videoCaptureDevice = new VideoCaptureDevice("Unregistered", "User", videoPictureBox);
                videoCaptureDevice.CaptureDeviceWindowChanged += new VideoCaptureDevice.CaptureDeviceWindowChangedHandler(videoCaptureDevice_CaptureDeviceWindowChanged);
                videoCaptureDevice.CreateCurrentControlCaptureWindow(0);
                videoCaptureDevice.ConnectCaptureWindowToCaptureDriver(0);
                if (videoCaptureDevice.IsDriverReadyToCapture) break;
                else
                {
                    videoCaptureDevice.FreeAllResources();
                    GC.Collect();
                    MessageBox.Show("There are no video capture devices connected to PC! Please connect and try again.", "Device Not Connected", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            videoCaptureDevice.CaptureMakeUserHitOKToCapture = false;
            videoCaptureDevice.CaptureAbortLeftMouse = false;
            videoCaptureDevice.CaptureAbortRightMouse = false;
            videoCaptureDevice.CaptureViaBackgroundTask = true;
            if (!videoCaptureDevice.HasVideoSourceDialog) videoSourceToolStripMenuItem.Enabled = false;
            if (!videoCaptureDevice.HasVideoFormatDialog) videoFormatToolStripMenuItem.Enabled = false;
            if (!videoCaptureDevice.HasVideoSourceDialog) videoCompressionToolStripMenuItem.Enabled = false;
            if (!videoCaptureDevice.HasVideoDisplayDialog) videoDisplayToolStripMenuItem.Enabled = false;
            if (!videoCaptureDevice.IsAudioHardwarePresent) audioFormatToolStripMenuItem.Enabled = false;
            if (!videoCaptureDevice.CanDeviceOverlay) overlayToolStripMenuItem.Enabled = false;
            stopVideoCaptureToolStripMenuItem.Enabled = false;
            InitializeSettings();
            statusUpdatingTimer.Start();
        }

        private void InitializeSettings()
        {
            //Main Form Settings.
            this.Size = Settings.Size;
            this.Location = Settings.Location;
            alwaysOnTopToolStripMenuItem.Checked = Settings.AlwaysOnTop;
            this.TopMost = Settings.AlwaysOnTop;
            useCaptureStartTime = Settings.UseCaptureStartTime;
            captureStartTime = Settings.CaptureStartTime;
            //Preview Settings.
            previewToolStripMenuItem.Checked = Settings.Preview;
            videoCaptureDevice.SetPreviewMode(Settings.Preview);
            //Overlay Settings.
            overlayToolStripMenuItem.Checked = Settings.Overlay;
            videoCaptureDevice.SetOverlayMode(Settings.Overlay);
            //Stretch Settings.
            stretchToolStripMenuItem.Checked = Settings.Stretch;
            if (Settings.Stretch) videoPictureBox.Dock = DockStyle.Fill; else videoPictureBox.Dock = DockStyle.None;
            videoCaptureDevice.StretchPreviewToCaptureWindow(Settings.Stretch);
            videoCaptureDevice.AdjustCaptureWindowSize();
            //Capture Audio Settings.
            captureAudioToolStripMenuItem.Checked = Settings.CaptureAudio;
            videoCaptureDevice.CaptureAudio = Settings.CaptureAudio;
            //Capture File Name Settings.
            videoCaptureDevice.CaptureFileName = Settings.CaptureFilePath;
            //Wave Format Settings.
            videoCaptureDevice.WaveBitsPerSample = Settings.BitsPerSample;
            videoCaptureDevice.WaveNumberOfChannels = Settings.WaveChannels;
            videoCaptureDevice.WaveSamplesPerSecond = Settings.SamplesPerSecond;
            //Use Capture Time Limit Settings.
            videoCaptureDevice.CaptureTimeLimitEnabled = Settings.UseCaptureTimeLimit;
            //Capture Time Limit Settings.
            videoCaptureDevice.CaptureTimeLimitInSeconds = Settings.CaptureTimeLimit;
            //Preview Frame Rate Settings.
            videoCaptureDevice.SetTimeIntervalBetweenPreviewFramesInMilliseconds((short)(1000 / Settings.PreviewFrameRate));
            //Capture Frame Rate Settings.
            videoCaptureDevice.TimeIntervalBetweenCapturingFramesInMicroseconds = 1000000 / Settings.CaptureFrameRate;
        }

        private void EnableOrDisableAfterStart()
        {
            setCaptureFileToolStripMenuItem.Enabled = false;
            allocateCaptureFileSpaceToolStripMenuItem.Enabled = false;
            setDIBFileToolStripMenuItem.Enabled = false;
            if (videoCaptureDevice.HasVideoSourceDialog) videoSourceToolStripMenuItem.Enabled = false;
            if (videoCaptureDevice.HasVideoFormatDialog) videoFormatToolStripMenuItem.Enabled = false;
            if (videoCaptureDevice.HasVideoSourceDialog) videoCompressionToolStripMenuItem.Enabled = false;
            if (videoCaptureDevice.HasVideoDisplayDialog) videoDisplayToolStripMenuItem.Enabled = false;
            if (videoCaptureDevice.IsAudioHardwarePresent) audioFormatToolStripMenuItem.Enabled = false;
            setPreviewFrameRateToolStripMenuItem.Enabled = false;
            previewToolStripMenuItem.Enabled = false;
            if (videoCaptureDevice.CanDeviceOverlay) overlayToolStripMenuItem.Enabled = false;
            stretchToolStripMenuItem.Enabled = false;
            startVideoCaptureToolStripMenuItem.Enabled = false;
            stopVideoCaptureToolStripMenuItem.Enabled = true;
            setCaptureFrameRateToolStripMenuItem.Enabled = false;
            setCaptureStartTimeToolStripMenuItem.Enabled = false;
            setCaptureTimeLimitToolStripMenuItem.Enabled = false;
            captureAudioToolStripMenuItem.Enabled = false;
            captureFramesToAVIFileToolStripMenuItem.Enabled = false;
            captureSingleFrameToClipboardToolStripMenuItem.Enabled = false;
            captureSingleFrameToDIBFileToolStripMenuItem.Enabled = false;
        }

        private void EnableOrDisableAfterStop()
        {
            setCaptureFileToolStripMenuItem.Enabled = true;
            allocateCaptureFileSpaceToolStripMenuItem.Enabled = true;
            setDIBFileToolStripMenuItem.Enabled = true;
            if (videoCaptureDevice.HasVideoSourceDialog) videoSourceToolStripMenuItem.Enabled = true;
            if (videoCaptureDevice.HasVideoFormatDialog) videoFormatToolStripMenuItem.Enabled = true;
            if (videoCaptureDevice.HasVideoSourceDialog) videoCompressionToolStripMenuItem.Enabled = true;
            if (videoCaptureDevice.HasVideoDisplayDialog) videoDisplayToolStripMenuItem.Enabled = true;
            if (videoCaptureDevice.IsAudioHardwarePresent) audioFormatToolStripMenuItem.Enabled = true;
            setPreviewFrameRateToolStripMenuItem.Enabled = true;
            previewToolStripMenuItem.Enabled = true;
            if (videoCaptureDevice.CanDeviceOverlay) overlayToolStripMenuItem.Enabled = true;
            stretchToolStripMenuItem.Enabled = true;
            startVideoCaptureToolStripMenuItem.Enabled = true;
            stopVideoCaptureToolStripMenuItem.Enabled = false;
            setCaptureFrameRateToolStripMenuItem.Enabled = true;
            setCaptureStartTimeToolStripMenuItem.Enabled = true;
            setCaptureTimeLimitToolStripMenuItem.Enabled = true;
            captureAudioToolStripMenuItem.Enabled = true;
            captureFramesToAVIFileToolStripMenuItem.Enabled = true;
            captureSingleFrameToClipboardToolStripMenuItem.Enabled = true;
            captureSingleFrameToDIBFileToolStripMenuItem.Enabled = true;
            alreadyCaptureTime = false;
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////MENU EVENTS//////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        ///////////////////////////////////////////////////FILE SUBMENU//////////////////////////////////////////////////
        private void setCaptureFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            setCaptureFileDialog.Title = "Set Capture File";
            setCaptureFileDialog.Filter = "Audio Video Interleaved (*.avi)|*.avi";
            setCaptureFileDialog.AddExtension = true;
            setCaptureFileDialog.CheckFileExists = false;
            setCaptureFileDialog.FileName = Settings.CaptureFilePath;
            DialogResult dialogResult = setCaptureFileDialog.ShowDialog();
            if (dialogResult == DialogResult.OK)
            {
                videoCaptureDevice.CaptureFileName = setCaptureFileDialog.FileName;
                Settings.CaptureFilePath = setCaptureFileDialog.FileName;
            }
        }

        private void allocateCaptureFileSpaceToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AllocateFileSpaceDialog allocateFileSpaceDialog = new AllocateFileSpaceDialog();
            allocateFileSpaceDialog.SetData(Settings.CaptureFileSize);
            DialogResult dialogResult = allocateFileSpaceDialog.ShowDialog();
            if (dialogResult == DialogResult.OK)
            {
                int captureFileSize;
                allocateFileSpaceDialog.GetData(out captureFileSize);
                videoCaptureDevice.AllocateSpaceForCaptureFile(captureFileSize * 1000000);
                Settings.CaptureFileSize = captureFileSize;
            }
        }

        private void setDIBFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            setDIBFileDialog.Title = "Set DIB File";
            setDIBFileDialog.Filter = "Bitmap Files (*.bmp)|*.bmp";
            setDIBFileDialog.AddExtension = true;
            setDIBFileDialog.CheckFileExists = false;
            setDIBFileDialog.FileName = Settings.DIBFilePath;
            DialogResult dialogResult = setDIBFileDialog.ShowDialog();
            if (dialogResult == DialogResult.OK) Settings.DIBFilePath = setDIBFileDialog.FileName;
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        /////////////////////////////////////////////////OPTIONS SUBMENU/////////////////////////////////////////////////
        private void videoSourceToolStripMenuItem_Click(object sender, EventArgs e) { videoCaptureDevice.ShowVideoSourceDialog(); }
        private void videoFormatToolStripMenuItem_Click(object sender, EventArgs e) { videoCaptureDevice.ShowVideoFormatDialog(); }
        private void videoCompressionToolStripMenuItem_Click(object sender, EventArgs e) { videoCaptureDevice.ShowVideoCompressionDialog(); }
        private void videoDisplayToolStripMenuItem_Click(object sender, EventArgs e) { videoCaptureDevice.ShowVideoDisplayDialog(); }

        private void audioFormatToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AudioFormatDialog audioFormatDialog = new AudioFormatDialog();
            audioFormatDialog.SetData(Settings.BitsPerSample, Settings.WaveChannels, Settings.SamplesPerSecond);
            DialogResult dialogResult = audioFormatDialog.ShowDialog();
            if (dialogResult == DialogResult.OK)
            {
                VideoCaptureDevice.BitsPerSample bitsPerSample;
                VideoCaptureDevice.WaveChannels waveChannels;
                int samplesPerSecond;
                audioFormatDialog.GetData(out bitsPerSample, out waveChannels, out samplesPerSecond);
                videoCaptureDevice.WaveBitsPerSample = bitsPerSample;
                videoCaptureDevice.WaveNumberOfChannels = waveChannels;
                videoCaptureDevice.WaveSamplesPerSecond = samplesPerSecond;
                Settings.BitsPerSample = bitsPerSample;
                Settings.WaveChannels = waveChannels;
                Settings.SamplesPerSecond = samplesPerSecond;
            }
        }

        private void setPreviewFrameRateToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PreviewFrameRateDialog previewFrameRateDialog = new PreviewFrameRateDialog();
            previewFrameRateDialog.SetData(Settings.PreviewFrameRate);
            DialogResult dialogResult = previewFrameRateDialog.ShowDialog();
            if (dialogResult == DialogResult.OK)
            {
                int previewFrameRate;
                previewFrameRateDialog.GetData(out previewFrameRate);
                videoCaptureDevice.SetTimeIntervalBetweenPreviewFramesInMilliseconds((short)(1000 / previewFrameRate));
                Settings.PreviewFrameRate = previewFrameRate;
            }
        }

        private void previewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (overlayToolStripMenuItem.Checked)
            {
                overlayToolStripMenuItem.Checked = false;
                previewToolStripMenuItem.Checked = true;
            }
            else previewToolStripMenuItem.Checked = !previewToolStripMenuItem.Checked;
            videoCaptureDevice.SetPreviewMode(previewToolStripMenuItem.Checked);
            Settings.Preview = previewToolStripMenuItem.Checked;
        }

        private void overlayToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (previewToolStripMenuItem.Checked)
            {
                previewToolStripMenuItem.Checked = false;
                overlayToolStripMenuItem.Checked = true;
            }
            else overlayToolStripMenuItem.Checked = !overlayToolStripMenuItem.Checked;
            videoCaptureDevice.SetOverlayMode(overlayToolStripMenuItem.Checked);
            Settings.Overlay = overlayToolStripMenuItem.Checked;
        }

        private void stretchToolStripMenuItem_Click(object sender, EventArgs e)
        {
            stretchToolStripMenuItem.Checked = !stretchToolStripMenuItem.Checked;
            if (stretchToolStripMenuItem.Checked) videoPictureBox.Dock = DockStyle.Fill; else videoPictureBox.Dock = DockStyle.None;
            videoCaptureDevice.StretchPreviewToCaptureWindow(stretchToolStripMenuItem.Checked);
            videoCaptureDevice.AdjustCaptureWindowSize();
            Settings.Stretch = stretchToolStripMenuItem.Checked;
        }

        private void alwaysOnTopToolStripMenuItem_Click(object sender, EventArgs e)
        {
            alwaysOnTopToolStripMenuItem.Checked = !alwaysOnTopToolStripMenuItem.Checked;
            this.TopMost = alwaysOnTopToolStripMenuItem.Checked;
            Settings.AlwaysOnTop = alwaysOnTopToolStripMenuItem.Checked;
        }

        /////////////////////////////////////////////////CAPTURE SUBMENU/////////////////////////////////////////////////
        private void startCaptureToolStripMenuItem_Click(object sender, EventArgs e)
        {
            EnableOrDisableAfterStart();
            videoCaptureDevice.StartCapturingAudioVideoToFile();
        }

        private void stopCaptureToolStripMenuItem_Click(object sender, EventArgs e)
        {
            EnableOrDisableAfterStop();
            videoCaptureDevice.AbortCapturingAudioVideo();
        }

        private void setCaptureFrameRateToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CaptureFrameRateDialog captureFrameRateDialog = new CaptureFrameRateDialog();
            captureFrameRateDialog.SetData(Settings.CaptureFrameRate);
            DialogResult dialogResult = captureFrameRateDialog.ShowDialog();
            if (dialogResult == DialogResult.OK)
            {
                int captureFrameRate;
                captureFrameRateDialog.GetData(out captureFrameRate);
                videoCaptureDevice.TimeIntervalBetweenCapturingFramesInMicroseconds = 1000000 / captureFrameRate;
                Settings.CaptureFrameRate = captureFrameRate;
            }
        }

        private void setCaptureStartTimeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CaptureStartTimeDialog captureStartTimeDialog = new CaptureStartTimeDialog();
            captureStartTimeDialog.SetData(Settings.UseCaptureStartTime, Settings.CaptureStartTime);
            DialogResult dialogResult = captureStartTimeDialog.ShowDialog();
            if (dialogResult == DialogResult.OK)
            {
                captureStartTimeDialog.GetData(out useCaptureStartTime, out captureStartTime);
                Settings.UseCaptureStartTime = useCaptureStartTime;
                Settings.CaptureStartTime = captureStartTime;
            }
        }

        private void setCaptureTimeLimitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CaptureTimeLimitDialog captureTimeLimitDialog = new CaptureTimeLimitDialog();
            captureTimeLimitDialog.SetData(Settings.UseCaptureTimeLimit, Settings.CaptureTimeLimit);
            DialogResult dialogResult = captureTimeLimitDialog.ShowDialog();
            if (dialogResult == DialogResult.OK)
            {
                bool useCaptureTimeLimit;
                int captureTimeLimit;
                captureTimeLimitDialog.GetData(out useCaptureTimeLimit, out captureTimeLimit);
                videoCaptureDevice.CaptureTimeLimitEnabled = useCaptureTimeLimit;
                videoCaptureDevice.CaptureTimeLimitInSeconds = captureTimeLimit;
                Settings.UseCaptureTimeLimit = useCaptureTimeLimit;
                Settings.CaptureTimeLimit = captureTimeLimit;
            }
        }

        private void captureAudioToolStripMenuItem_Click(object sender, EventArgs e)
        {
            captureAudioToolStripMenuItem.Checked = !captureAudioToolStripMenuItem.Checked;
            videoCaptureDevice.CaptureAudio = captureAudioToolStripMenuItem.Checked;
            Settings.CaptureAudio = captureAudioToolStripMenuItem.Checked;
        }

        private void captureSingleFrameToAVIFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            videoCaptureDevice.SetSingleFrameCapturingMode();
            CaptureFramesDialog captureFramesDialog = new CaptureFramesDialog(videoCaptureDevice);
            captureFramesDialog.Clear();
            DialogResult dialogResult = captureFramesDialog.ShowDialog();
            videoCaptureDevice.CancelSingleFrameCapturingMode();
        }

        private void captureSingleFrameToClipboardToolStripMenuItem_Click(object sender, EventArgs e)
        {
            videoCaptureDevice.RetrieveAndShowSingleFrameOverlayPreviewNotAltered();
            videoCaptureDevice.CaptureSingleFrameToClipboard();
        }

        private void captureSingleFrameToDIBFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            videoCaptureDevice.RetrieveAndShowSingleFrameOverlayPreviewNotAltered();
            videoCaptureDevice.SaveCurrentFrameToDIBFile(Settings.DIBFilePath);
        }

        /////////////////////////////////////////////////HELP SUBMENU/////////////////////////////////////////////////
        private void helpTopicsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start(Application.StartupPath + "\\SoftCollectionVideoCaptureLibraryForNET.chm");
        }

        private void visitWebSiteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.soft-collection.com");
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutBox about = new AboutBox();
            about.ShowDialog();
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /////////////////////////////////////////////////MAIN FORM EVENTS////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        private void MainFrm_LocationChanged(object sender, EventArgs e)
        {
            if (this.WindowState == FormWindowState.Normal) Settings.Location = this.Location;
        }

        private void MainFrm_Resize(object sender, EventArgs e)
        {
            videoCaptureDevice.AdjustCaptureWindowSize();
            if (this.WindowState == FormWindowState.Normal) Settings.Size = this.Size;
        }

        private void MainFrm_FormClosed(object sender, FormClosedEventArgs e)
        {
            videoCaptureDevice.FreeAllResources();
            GC.Collect();
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        ///////////////////////////////////////////////////OTHER EVENTS//////////////////////////////////////////////////
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        private void videoCaptureDevice_CaptureDeviceWindowChanged(object sender, CaptureDeviceWindowChangedEventArgs e)
        {
            videoPictureBox.Width = e.Size.Width;
            videoPictureBox.Height = e.Size.Height;
            AdjustVideoWindowPlace();
        }

        private void AdjustVideoWindowPlace()
        {
            videoPictureBox.Left = Math.Max(0, (videoPanel.Width - videoPictureBox.Width) / 2);
            videoPictureBox.Top = Math.Max(0, (videoPanel.Height - videoPictureBox.Height) / 2);
        }

        private void statusUpdatingTimer_Tick(object sender, EventArgs e)
        {
            if (videoCaptureDevice.IsCaptureInProgress)
            {
                recordingToolStripStatusLabel.Text = "Recording...";
                this.recordingToolStripStatusLabel.Image = global::VideoCaptureLibraryDemo.Properties.Resources.ledyellow;
            }
            else
            {
                recordingToolStripStatusLabel.Text = "Not recording";
                this.recordingToolStripStatusLabel.Image = global::VideoCaptureLibraryDemo.Properties.Resources.ledgray;
                EnableOrDisableAfterStop();
            }

            if (videoCaptureDevice.IsNowPreviewingVideo)
            {
                previewingToolStripStatusLabel.Text = "Previewing...";
                this.previewingToolStripStatusLabel.Image = global::VideoCaptureLibraryDemo.Properties.Resources.ledred;
            }
            else
            {
                previewingToolStripStatusLabel.Text = "Not previewing";
                this.previewingToolStripStatusLabel.Image = global::VideoCaptureLibraryDemo.Properties.Resources.ledgray;
            }

            if (videoCaptureDevice.IsNowOverlayingVideo)
            {
                overlayingToolStripStatusLabel.Text = "Overlaying...";
                this.overlayingToolStripStatusLabel.Image = global::VideoCaptureLibraryDemo.Properties.Resources.ledgreen;
            }
            else
            {
                overlayingToolStripStatusLabel.Text = "Not overlaying...";
                this.overlayingToolStripStatusLabel.Image = global::VideoCaptureLibraryDemo.Properties.Resources.ledgray;
            }

            TimeSpan timeDifference = captureStartTime.Subtract(DateTime.Now);
            int timeDifferenceInSeconds = Math.Abs(timeDifference.Days * 86400 + timeDifference.Hours * 3600 + timeDifference.Minutes * 60 + timeDifference.Seconds);
            if ((!alreadyCaptureTime) && (timeDifferenceInSeconds < 1) && (!videoCaptureDevice.IsCaptureInProgress) && (useCaptureStartTime))
            {
                EnableOrDisableAfterStart();
                videoCaptureDevice.StartCapturingAudioVideoToFile();
                alreadyCaptureTime = true;
            }
        }
    }
}